/* gen.c
 *
 * GENERATE CASES FOR THE "DEMO" DATASET AND WRITE THEM TO STANDARD OUTPUT. 
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>


/* ADJUSTABLE PARAMETERS. */

#define Seed 1		/* Random number seed to use */
#define N_cases 2048	/* Number of cases to generate */


/* GENERATE RANDOM VARIATE FROM UNIFORM(0,1) DISTRIBUTION. */

double ranu(void)
{
  return (rand()+0.5)/32768.0;
}


/* GENERATE RANDOM VARIATE FROM EXPONENTIAL DISTRIBUTION. */

double rane(void)
{ 
  return -log(ranu());
}


/* GENERATE RANDOM VARIATE FROM STANDARD NORMAL DISTRIBUTION. */

double rann(void)
{
  double a, b;

  a = ranu();
  b = ranu();

  return cos(2.0*M_PI*a) * sqrt(-2.0*log(b));
}


/* MAIN PROGRAM. */

void main(void)
{
  enum { Male, Female } sex;
  double age;
  double income;
  int siblings;
  enum { Pink, Blue, Red, Green, Purple } colour;
  int working;
  int childlike;
  double C, U;
  int i;

  static char *sex_name[] = { " male ", "female" };
  static char *colour_name[] = { "pink", "blue", "red", "green", "purple" };

  srand(Seed);

  for (i = 0; i<N_cases; i++)
  {
    sex = ranu()<0.53 ? Female : Male;

    age = rann() * (sex==Female ? 40 : 30);
    if (age<0) age = -age;

    siblings = (int) (rane() * 3*(1+age)/(3+age));

    income = rane() * age*100;

    working = ranu()<(sex==Male ? 0.9/((1+exp(-(age-20)))*(1+exp((age-60)/5)))
                                : 0.8/((1+exp(-(age-20)))*(1+exp((age-65)/5))));

    if (working)
    { C = sex==Female ? 30000 : 40000;
      income += rane() * (C - 5000*siblings/(5+siblings)) 
                       / (1 + exp(-(age-25)/5));
    }
    
    childlike = ranu() < 1/(1+exp(age-10));

    U = ranu();

    colour = U<0.08 ? Pink 
           : U<0.08+0.27 ? Blue 
           : U<0.08+0.27+0.31 ? Red
           : U<0.08+0.27+0.31+0.24 ? Green
           : Purple;

    if (childlike && sex==Female)
    { if (ranu()<0.9) colour = Pink;
    }
    if (childlike && sex==Male)
    { if (ranu()<0.9) colour = Blue;
    }
    if (!childlike)
    { if (ranu()<1/(1+exp(-(income-80000)/10000))) colour = Purple;
    }

    printf(" %6s  %5.1lf  %2d  %6.0lf  %s\n", sex_name[sex], age, siblings,
      income, colour_name[colour]);
  }

  exit(0);
}
